@@ -982,6 +982,9 @@ class Normalizer(object):
             location=self._get_current_location(),
         )
 
+        if not self.strict_mode:
+            ret['uuid'] = c_id
+
         for key in (
                 'status',
                 'cluster_template_id',
@@ -991,7 +994,8 @@ class Normalizer(object):
                 'create_timeout',
                 'node_count',
                 'name'):
-            ret[key] = coe_cluster.pop(key)
+            if key in coe_cluster:
+                ret[key] = coe_cluster.pop(key)
 
         ret['properties'] = coe_cluster
         return ret
